/*
 * Copyright (c) 2018-2019 ARM Limited
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "pwm_ip6512_drv.h"

/**
 * \brief PWM register map structure
 */
struct pwm_reg_map_t {
    volatile uint32_t pwmcr;    /*!< Offset: 0x000 Control Register */
    volatile uint32_t pwmpr;    /*!< Offset: 0x004 Period Register */
    volatile uint32_t pwmhr;    /*!< Offset: 0x008 Hightime Register */
    volatile uint32_t reserved; /*!< Offset: 0x00C Reserved */
    volatile uint32_t pwmei;    /*!< Offset: 0x010 Enable Interrupt */
    volatile uint32_t pwmdi;    /*!< Offset: 0x014 Disable Interrupt */
    volatile uint32_t pwmri;    /*!< Offset: 0x018 Read Intr Enable Reg */
    volatile uint32_t pwmis;    /*!< Offset: 0x01C Read Intr Status */
};

#define PWM_PWMCR_EN_MASK    (0x1u) /*!< PWM control register enable mask */
#define PWM_PWMEI_EN_MASK    (0x1u) /*!< PWM enable interrupt register mask */
#define PWM_PWMDI_DIS_MASK   (0x1u) /*!< PWM disable interrupt register mask */
#define PWM_INTR_MASK        (0x1u) /*!< PWM interrupt mask */

static void _pwm_enable(struct pwm_reg_map_t* p_pwm)
{
    p_pwm->pwmcr |= PWM_PWMCR_EN_MASK;
}

static void _pwm_disable(struct pwm_reg_map_t* p_pwm)
{
    p_pwm->pwmcr &= ~PWM_PWMCR_EN_MASK;
}

enum pwm_error_t pwm_init(struct pwmout_dev_t* dev, uint32_t pwm_clk_in_hz)
{
    struct pwm_reg_map_t* p_pwm = (struct pwm_reg_map_t*) dev->cfg->base;

    if(dev->data->state == pwm_initialized) {
        return pwm_err_already_init;
    }

    if(pwm_clk_in_hz == 0) {
        return pwm_err_invalid_arg;
    }

    dev->data->state = pwm_initialized;

    pwm_set_period(dev, dev->cfg->period_in_ticks);

    pwm_set_high_time(dev, dev->cfg->high_time_in_ticks);

    _pwm_enable(p_pwm);

    return pwm_err_none;
}

void pwm_uninit(struct pwmout_dev_t* dev)
{
    struct pwm_reg_map_t* p_pwm = (struct pwm_reg_map_t*) dev->cfg->base;

    _pwm_disable(p_pwm);

    dev->data->state = pwm_uninitialized;

    return;
}

void pwm_enable(struct pwmout_dev_t* dev)
{
    struct pwm_reg_map_t* p_pwm = (struct pwm_reg_map_t*) dev->cfg->base;

    _pwm_enable(p_pwm);

    return;
}

void pwm_disable(struct pwmout_dev_t* dev)
{
    struct pwm_reg_map_t* p_pwm = (struct pwm_reg_map_t*) dev->cfg->base;

    _pwm_disable(p_pwm);

    return;
}

void pwm_set_period(struct pwmout_dev_t* dev, uint32_t period_in_ticks)
{
    struct pwm_reg_map_t* p_pwm = (struct pwm_reg_map_t*) dev->cfg->base;

    p_pwm->pwmpr = period_in_ticks;

    return;
}

uint32_t pwm_get_period(struct pwmout_dev_t* dev)
{
    struct pwm_reg_map_t* p_pwm = (struct pwm_reg_map_t*) dev->cfg->base;

    return p_pwm->pwmpr;
}

void pwm_set_high_time(struct pwmout_dev_t* dev, uint32_t high_time_in_ticks)
{
    struct pwm_reg_map_t* p_pwm = (struct pwm_reg_map_t*) dev->cfg->base;

    p_pwm->pwmhr = high_time_in_ticks;

    return;
}

uint32_t pwm_get_high_time(struct pwmout_dev_t* dev)
{
    struct pwm_reg_map_t* p_pwm = (struct pwm_reg_map_t*) dev->cfg->base;

    return p_pwm->pwmhr;
}

void pwm_enable_interrupt(struct pwmout_dev_t* dev)
{
    struct pwm_reg_map_t* p_pwm = (struct pwm_reg_map_t*) dev->cfg->base;

    p_pwm->pwmei = PWM_PWMEI_EN_MASK;

    return;
}

void pwm_disable_interrupt(struct pwmout_dev_t* dev)
{
    struct pwm_reg_map_t* p_pwm = (struct pwm_reg_map_t*) dev->cfg->base;

   p_pwm->pwmdi = PWM_PWMDI_DIS_MASK;
   return;
}


bool pwm_is_interrupt_enabled(struct pwmout_dev_t* dev)
{
    struct pwm_reg_map_t* p_pwm = (struct pwm_reg_map_t*) dev->cfg->base;

    return (bool)(p_pwm->pwmri & PWM_INTR_MASK);
}


bool pwm_get_interrupt_status(struct pwmout_dev_t* dev)
{
    struct pwm_reg_map_t* p_pwm = (struct pwm_reg_map_t*) dev->cfg->base;

    return (bool)(p_pwm->pwmis & PWM_INTR_MASK);
}
