/*
 * Copyright (c) 2018-2019 ARM Limited
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * \file pwm_ip6512_drv.h
 * \brief Generic driver for PWM (Candence IP6512).
 */

#ifndef __PWM_IP6512_DRV_H__
#define __PWM_IP6512_DRV_H__

#include <stdint.h>
#include <stdbool.h>

#ifdef __cplusplus
extern "C" {
#endif

/**
 * \brief PWM state types
 */
enum pwm_state_t {
    pwm_uninitialized = 0x0u,
    pwm_initialized   = 0x1u,
};

/**
 * \brief PWM device configuration structure
 */
struct pwm_dev_cfg_t {
    const uint32_t base;         /*!< PWM base address */
    uint32_t period_in_ticks;    /*!< Period of PWM cycle in ticks */
    uint32_t high_time_in_ticks; /*!< High time of PWM cycle in ticks */
};

/**
 * \brief PWM device data structure
 */
struct pwm_dev_data_t {
    enum pwm_state_t state;  /*!< PWM state */
};

/**
 * \brief PWM device structure
 */
struct pwmout_dev_t {
    const struct pwm_dev_cfg_t* const cfg;  /*!< PWM configuration */
    struct pwm_dev_data_t* const data;      /*!< PWM data */
};

/**
 * \brief PWM error enumeration
 */
enum pwm_error_t {
    pwm_err_none = 0,     /*!< No error */
    pwm_err_invalid_arg,  /*!< Error invalid input argument */
    pwm_err_not_init,     /*!< Error pwm is not initialized */
    pwm_err_already_init, /*!< Error pwm is already initialized */
};

enum pwm_period_special_val_t {
    pwm_period_cont_high = 0x0,       /*!< pwm_output continually high. */
    pwm_period_cont_low  = 0xFFFFFFFF /*!< pwm_output continually low.  */
};

/**
 * \brief Initializes PWM device
 *
 * \param[in] dev             PWM device struct \ref pwmout_dev_t
 * \param[in] pwm_clk_in_hz   PWM clock used by the device.
 *
 * It uses the default period and high time to configure PWM. Enables PWM.
 *
 * \return Returns error code as specified in \ref pwm_error_t
 *
 * \note This API should be called before calling any of the below PWM APIs
 * \note This function doesn't check if dev is NULL.
 */
enum pwm_error_t pwm_init(struct pwmout_dev_t* dev, uint32_t pwm_clk_in_hz);

/**
 * \brief Uninitializes PWM device
 *
 * \param[in] dev   PWM device struct \ref pwmout_dev_t
 *
 * The function uninitializes the PWM device and disables it.
 *
 * \note This function doesn't check if dev is NULL.
 */
void pwm_uninit(struct pwmout_dev_t* dev);

/**
 * \brief Enables PWM device
 *
 * \param[in] dev  PWM device struct \ref pwmout_dev_t
 *
 * The function sets the enable bit in the PWM device.
 *
 * \note This function doesn't check if dev is NULL.
 */
void pwm_enable(struct pwmout_dev_t* dev);

/**
 * \brief Disables PWM device
 *
 * \param[in] dev  PWM device struct \ref pwmout_dev_t
 *
 * The function resets the enable bit in the PWM device.
 *
 * \note This function doesn't check if dev is NULL.
 */
void pwm_disable(struct pwmout_dev_t* dev);

/**
 * \brief Sets the PWM period.
 *
 * \param[in] dev              PWM device struct \ref pwmout_dev_t
 * \param[in] period_in_ticks  New period (in clock ticks)
 *
 * \note The minimum and maximum values have special significance.
 *       \ref pwm_period_special_val_t
 * \note This function doesn't check if dev is NULL.
 */
void pwm_set_period(struct pwmout_dev_t* dev, uint32_t period_in_ticks);

/**
 * \brief Returns the PWM period.
 *
 * \param[in] dev         PWM device struct \ref pwmout_dev_t
 *
 * \return Returns the PWM period (in clock ticks)
 *
 * \note The minimum and maximum values have special significance.
 *       \ref pwm_period_special_val_t
 * \note This function doesn't check if dev is NULL.
 */
uint32_t pwm_get_period(struct pwmout_dev_t* dev);

/**
 * \brief Sets the PWM high time.
 *
 * \param[in] dev                 PWM device struct \ref pwmout_dev_t
 * \param[in] high_time_in_ticks  High time of PWM pulse (in clock ticks)
 *
 * \return Returns error code as specified in \ref pwm_error_t
 *
 * \note High time value should be less than the period value, otherwise the
 *       pwm_output will be continually high.
 * \note This function doesn't check if dev is NULL.
 */
void pwm_set_high_time(struct pwmout_dev_t* dev, uint32_t high_time_in_ticks);

/**
 * \brief Returns the PWM high time.
 *
 * \param[in] dev         PWM device struct \ref pwmout_dev_t
 *
 * \return Returns the PWM high time (in clock ticks)
 *
 * \note This function doesn't check if dev is NULL.
 */
uint32_t pwm_get_high_time(struct pwmout_dev_t* dev);

/**
 * \brief Enables the PWM interrupt
 *
 * \param[in] dev         PWM device struct \ref pwmout_dev_t
 *
 * IP can generate interrupts every time the period register overflows
 *
 * \note This function doesn't check if dev is NULL.
 */
void pwm_enable_interrupt(struct pwmout_dev_t* dev);

/**
 * \brief Disables the PWM interrupt
 *
 * \param[in] dev         PWM device struct \ref pwmout_dev_t
 *
 * \note This function doesn't check if dev is NULL.
 */
void pwm_disable_interrupt(struct pwmout_dev_t* dev);

/**
 * \brief Returns the Interrupt enable status
 *
 * \param[in] dev         PWM device struct \ref pwmout_dev_t
 *
 * \return Returns true if the interrupt is enabled or false if it is disabled
 *
 * \note This function doesn't check if dev is NULL.
 */
bool pwm_is_interrupt_enabled(struct pwmout_dev_t* dev);

/**
 * \brief Returns the Interrupt status
 *
 * \param[in] dev         PWM device struct \ref pwmout_dev_t
 *
 * \return Returns true if the interrupt is active or false otherwise
 *
 * \note Calling this function resets the interrupt status register
 * \note Returns the current state of the PWM Interrupt, and then sets
 *       the bit low
 * \note This function doesn't check if dev is NULL.
 */
bool pwm_get_interrupt_status(struct pwmout_dev_t* dev);

#ifdef __cplusplus
}
#endif
#endif /* __PWM_IP6512_DRV_H__ */
